@@ -8,11 +8,15 @@ BEGIN {
     use Exporter ();
     use List::Util qw( min );
     our ($VERSION, @ISA, @EXPORT_OK);
-    $VERSION     = '0.01';
+    $VERSION     = '0.05';
     @ISA         = qw(Exporter);
     @EXPORT_OK   = qw(
         social_security_benefits
+        social_security_worksheet_data
+        pp_ssbw
         qualified_dividends_capital_gains_tax
+        pp_qdcgtw
+        decimal_lines
     );
 }
 
@@ -24,12 +28,19 @@ Business::Tax::US::Form_1040::Worksheets - IRS Form 1040 worksheets calculations
 
     use Business::Tax::US::Form_1040::Worksheets qw(
         social_security_benefits
+        social_security_worksheet_data
         qualified_dividends_capital_gains_tax
+        pp_qdcgtw
+        decimal_lines
     );
 
     $benefits = social_security_benefits( $inputs );
 
-    $lines = qualified_dividends_capital_gains_tax($inputs);
+    $worksheet_data = social_security_worksheet_data( $inputs );
+
+    $lines = qualified_dividends_capital_gains_tax( $inputs );
+
+    pp_qdcgtw($lines);
 
 =head1 DESCRIPTION
 
@@ -180,6 +191,44 @@ our %params = (
 );
 
 sub social_security_benefits {
+    my ($inputs) = @_;
+    my $rv = _social_security_benefits_engine($inputs);
+    return $rv->{taxable_benefits};
+}
+
+=head2 C<social_security_worksheet_data()>
+
+=over 4
+
+=item * Purpose
+
+Calculate data needed for the purpose of completing all entries (except
+checkboxes) on the Social Security Benefits Worksheet (For filing year 2023,
+these would be lines 6a and 6b on that form.)
+
+    my $worksheet_data = social_security_worksheet_data( $inputs );
+
+=item * Arguments
+
+The same, single hash reference which is supplied to
+C<social_security_benefits()> (I<q.v.> above).
+
+=item * Return Value
+
+Reference to an array holding the data to be entered on lines 1-18 of the SSBW.
+The indexes to the elements of that array correspond to those line numbers on the SSBW.
+
+=back
+
+=cut
+
+sub social_security_worksheet_data {
+    my ($inputs) = @_;
+    my $rv = _social_security_benefits_engine($inputs);
+    return $rv->{worksheet_data};
+}
+
+sub _social_security_benefits_engine {
     my ($inputs) = @_;
     croak "Argument to social_security_benefits() must be hashref"
         unless ref($inputs) eq 'HASH';
@@ -215,6 +264,7 @@ sub social_security_benefits {
     $data->{status} = $inputs->{status};
     my $filing_year = $inputs->{filing_year}; # TODO add test for numeric; required element
     my @lines = (undef, (undef) x $params{ssb}{$filing_year}{worksheet_line_count});
+    my $formatted_lines;
     $lines[1] = $data->{box5};
     $lines[2] = $lines[1] * $params{ssb}{$filing_year}{ssa_percentage};
     $lines[3] =
@@ -241,18 +291,34 @@ sub social_security_benefits {
         $data->{s1l20} +
         $data->{s1l23} +
         $data->{s1l25};
-    return 0 if ! ($lines[6] < $lines[5]);
+    if (! ($lines[6] < $lines[5]) ) {
+        $formatted_lines = decimal_lines(\@lines);
+        return {
+          taxable_benefits => 0,
+          worksheet_data => $formatted_lines,
+        };
+    }
     $lines[7] = $lines[5] - $lines[6];
     if ($data->{status} eq 'married_sep') {
         $lines[16] = $lines[7] * $params{ssb}{$filing_year}{percentage_b};
         $lines[17] = $lines[1] * $params{ssb}{$filing_year}{percentage_c};
         $lines[18] = min($lines[16], $lines[17]);
-        return $lines[18];
+        $formatted_lines = decimal_lines(\@lines);
+        return {
+            taxable_benefits => $lines[18],
+            worksheet_data => $formatted_lines,
+        };
     }
     $lines[8] = $data->{status} eq 'married'
         ? $params{ssb}{$filing_year}{married_amt_a}
         : $params{ssb}{$filing_year}{single_amt_a};
-    return 0 unless $lines[8] < $lines[7];
+    unless ($lines[8] < $lines[7]) {
+        $formatted_lines = decimal_lines(\@lines);
+        return {
+            taxable_benefits => 0,
+            worksheet_data => $formatted_lines,
+        };
+    }
     $lines[9] = $lines[7] - $lines[8];
     $lines[10] = $data->{status} eq 'married'
         ? $params{ssb}{$filing_year}{married_amt_b}
@@ -267,7 +333,99 @@ sub social_security_benefits {
     $lines[16] = $lines[14] + $lines[15];
     $lines[17] = $lines[1] * $params{ssb}{$filing_year}{percentage_c};
     $lines[18] = min($lines[16], $lines[17]);
-    return sprintf("%.2f" => $lines[18]);
+    $formatted_lines = decimal_lines(\@lines);
+    return {
+        taxable_benefits => $lines[18],
+        worksheet_data => $formatted_lines,
+    };
+}
+
+=head2 C<pp_ssbw()>
+
+=over 4
+
+=item * Purpose
+
+Pretty-print ('pp' for short) the results of
+C<social_security_worksheet_data()> for easier transcription to printed
+worksheet.
+
+    pp_ssbw($results);
+
+=item * Arguments
+
+The array reference which is the return value of
+C<social_security_worksheet_data()>.  Required.
+
+=item * Return Value
+
+Implicitly returns true value upon success.
+
+=item * Comment
+
+In a future version of this library, this function may take a second argument
+which presumably will be a string holding the path to an output file.  For
+now, the function simply prints to C<STDOUT>.
+
+=back
+
+=cut
+
+sub _compose_worksheet_line {
+    my ($line_number, $formatting, $text, $result) = @_;
+    my $line = sprintf("$formatting" => (
+        $line_number,
+        $text,
+        $line_number,
+        $result,
+    ) );
+    return $line;
+}
+
+sub pp_ssbw {
+    my ($results) = @_;
+    croak "First argument to pp_ssbw() must be array reference"
+        unless ref($results) eq 'ARRAY';
+    my @output = ();
+    my $line_number = 0;
+    my $one_wide_format     = "% 2s. %-40s  % 2s. % 12.2f";
+    my $two_wide_format     = "% 2s. %-52s        % 2s. % 12.2f";
+    my @f = (undef, $one_wide_format, $two_wide_format);
+
+    my $lines = [
+        undef,
+        { formatting => $f[1], text => "Enter sum of 1099s, box 5" },
+        { formatting => $f[2], text => "Line 1 x 50%" },
+        { formatting => $f[2], text => "Sum of 1040 lines 1z 2b 3b 4b 5b 7 8" },
+        { formatting => $f[2], text => "1040 line 2a" },
+        { formatting => $f[2], text => "Line 2 + Line 3 + Line 4" },
+        { formatting => $f[2], text => "Schedule 1, sum lines 11-20, 23, 25" },
+        { formatting => $f[2], text => "Line 5 - Line 6 (or 0)" },
+        { formatting => $f[2], text => "Status (1)" },
+        { formatting => $f[2], text => "Line 7 - Line 8 (or 0)" },
+        { formatting => $f[2], text => "Status (2)" },
+        { formatting => $f[2], text => "Line 9 - Line 10" },
+        { formatting => $f[2], text => "Smaller of lines 9 or 10" },
+        { formatting => $f[2], text => "Line 12 x 50%" },
+        { formatting => $f[2], text => "Smaller of lines 2 or 13" },
+        { formatting => $f[2], text => "Line 11 x 85%" },
+        { formatting => $f[2], text => "Line 14 + Line 15" },
+        { formatting => $f[2], text => "Line 1 x 85%" },
+        { formatting => $f[2], text => "Smaller of lines 16 or 17" },
+    ];
+
+    for (my $i = 0; $i <= $#{$results} -1 ; $i++) {
+        my $j = $i + 1;
+        push @output, _compose_worksheet_line(
+            $j,
+            $lines->[$j]->{formatting},
+            $lines->[$j]->{text},
+            $results->[$j],
+        );
+    }
+
+    say $_ for @output;
+    return 1;
 }
 
 =head2 C<qualified_dividends_capital_gains_tax()>
@@ -280,7 +438,7 @@ B<Partial calculation> of taxes due per the QDCGTW for the purpose of
 entering the amount of such taxes due on IRS Form 1040.  (For filing
 year 2023, these would be Form 1040 line 16.)
 
-    my $lines = qualified_dividends_capital_gains_tax($inputs);
+    my $lines = qualified_dividends_capital_gains_tax( $inputs );
 
 =item * Arguments
 
@@ -435,6 +593,128 @@ sub qualified_dividends_capital_gains_tax {
     return \@formatted_lines;
 }
 
+=head2 C<pp_qdcgtw()>
+
+=over 4
+
+=item * Purpose
+
+Pretty-print ('pp' for short) the results of
+C<qualified_dividends_capital_gains_tax()> for easier transcription to printed
+worksheet.
+
+    pp_qdcgtw($results);
+
+=item * Arguments
+
+The array reference which is the return value of
+C<qualified_dividends_capital_gains_tax()>.  Required.
+
+=item * Return Value
+
+Implicitly returns true value upon success.
+
+=item * Comment
+
+In a future version of this library, this function may take a second argument
+which presumably will be a string holding the path to an output file.  For
+now, the function simply prints to C<STDOUT>.
+
+=back
+
+=cut
+
+sub pp_qdcgtw {
+    my ($results) = @_;
+    croak "First argument to pp_qdcgtw() must be array reference"
+        unless ref($results) eq 'ARRAY';
+    my @output = ();
+    my $line_number = 0;
+    my $one_wide_format     = "% 2s. %-28s  % 2s. % 12.2f";
+    my $two_wide_format     = "% 2s. %-40s        % 2s. % 12.2f";
+    my $three_wide_format   = "% 2s. %-52s              % 2s. % 12.2f";
+    my @f = (undef, $one_wide_format, $two_wide_format, $three_wide_format);
+
+    my $lines = [
+        undef,
+        { formatting => $f[2], text => "Enter Form 1040, line 15" },
+        { formatting => $f[1], text => "Enter Form 1040, line 3a" },
+        { formatting => $f[1], text => "Sched. D/Form 1040, line 7" },
+        { formatting => $f[1], text => "Line 2 - Line 3" },
+        { formatting => $f[2], text => "Line 1 - Line 4" },
+        { formatting => $f[2], text => "Filing status amount (1)" },
+        { formatting => $f[2], text => "Smaller of lines 1 or 6" },
+        { formatting => $f[2], text => "Smaller of lines 5 or 7" },
+        { formatting => $f[2], text => "Line 7 - Line 8" },
+        { formatting => $f[2], text => "Smaller of lines 1 or 4" },
+        { formatting => $f[2], text => "Line 9 amount" },
+        { formatting => $f[2], text => "Line 10 - Line 11" },
+        { formatting => $f[2], text => "Filing status amount (2)" },
+        { formatting => $f[2], text => "Smaller of lines 1 or 13" },
+        { formatting => $f[2], text => "Line 5 + Line 9" },
+        { formatting => $f[2], text => "Line 14 - Line 15" },
+        { formatting => $f[2], text => "Smaller of lines 12 or 16" },
+        { formatting => $f[3], text => "Line 17 x 15%" },
+        { formatting => $f[2], text => "Line 9 + Line 17" },
+        { formatting => $f[2], text => "Line 10 - Line 19" },
+        { formatting => $f[3], text => "Line 20 x 20%" },
+    ];
+
+    for (my $i = 0; $i <= $#{$results} -1 ; $i++) {
+        my $j = $i + 1;
+        push @output, _compose_worksheet_line(
+            $j,
+            $lines->[$j]->{formatting},
+            $lines->[$j]->{text},
+            $results->[$j],
+        );
+    }
+
+    say $_ for @output;
+    return 1;
+}
+
+=head2 C<decimal_lines()>
+
+=over 4
+
+=item * Purpose
+
+This is a helper subroutine used within both this module and the test suite to
+ensure that all final monetary data is appropriately reported to two decimal
+places.
+
+=item * Arguments
+
+    my $formatted_lines = decimal_lines($lines);
+
+Single array reference holding a list of the values calculated for the various
+lines in a worksheet.
+
+=item * Return Value
+
+Single array reference holding a list of values prepared for entry into the
+worksheets to two decimal places (except for where the value is zero (C<0>).
+Values that are undefined remain so.
+
+=back
+
+=cut
+
+sub decimal_lines {
+    my $lines = shift;
+    my @formatted_lines = ();
+    for my $l (@{$lines}) {
+        if (! defined $l or $l eq '0') {
+            push @formatted_lines, $l;
+        }
+        else {
+            push @formatted_lines, sprintf("%.2f" => $l);
+        }
+    }
+    return \@formatted_lines;
+}
+
 =head1 AUTHOR
 
     James E Keenan
